package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.entity.KnowledgeDataEntity;
import com.mh.user.service.KnowledgeDataService;
import io.jsonwebtoken.lang.Assert;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 知识库管理
 * @date 2024-06-26 14:39:24
 */
@RestController
@RequestMapping("/knowledge")
public class KnowledgeDataController {

    @Resource
    private KnowledgeDataService knowledgeDataService;

    @GetMapping("/query")
    public HttpResult queryKnowledgeData(@RequestParam("pageNum") int pageNum, @RequestParam("pageSize") int pageSize) {
        PageRequest pageRequest = new PageRequest();
        pageRequest.setPageNum(pageNum);
        pageRequest.setPageSize(pageSize);
        return HttpResult.ok(knowledgeDataService.queryKnowledgeData(pageRequest));
    }

    @GetMapping("/{id}")
    public HttpResult detail(@PathVariable(name = "id") Long id) {
        KnowledgeDataEntity knowledgeData = knowledgeDataService.getById(id);
        Assert.notNull(knowledgeData, "该文章已被删除");
        return HttpResult.ok(knowledgeData);
    }

    @PostMapping("/update")
    public HttpResult updateData(@Validated @RequestBody KnowledgeDataEntity knowledgeData) {
        try {
            knowledgeDataService.updateData(knowledgeData);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return HttpResult.ok();
    }

    @PostMapping("/insert")
    public HttpResult insertKnowledgeData(@Validated @RequestBody KnowledgeDataEntity knowledgeData) {
        try {
            knowledgeDataService.insertKnowledgeData(knowledgeData);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return HttpResult.ok();
    }

}
