package com.mh.user.mapper;

import com.mh.user.dto.EnergyPreEchartDataDTO;
import com.mh.user.dto.EnergyPreTopDataDTO;
import com.mh.user.entity.HistoryDataPre;
import org.apache.ibatis.annotations.*;
import org.springframework.security.core.parameters.P;
import tk.mybatis.mapper.common.BaseMapper;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 预测历史数据mapper
 * @date 2024-05-09 10:01:46
 */
@Mapper
public interface HistoryDataPreMapper extends BaseMapper<HistoryDataPre> {

    @Results(id ="rs_train_data",value ={
            @Result(column = "env_min_temp", property = "envMinTemp"),
            @Result(column = "env_max_temp", property = "envMaxTemp"),
            @Result(column = "water_value", property = "waterValue"),
            @Result(column = "elect_value", property = "electValue"),
            @Result(column = "water_level", property = "waterLevel"),
            @Result(column = "people_num", property = "peopleNum")
    })
    @Select("select  env_min_temp, env_max_temp, water_value, elect_value, water_level, people_num from history_data_pre where building_id = #{buildingId} " +
            " and env_min_temp > 0 and env_max_temp >0 and water_value  > 0 and elect_value > 0 and water_level >0 " +
            " order by cur_date ")
    List<HistoryDataPre> getTrainData(@Param("buildingId") String buildingId);

    @Results(id ="rs_recent_data",value ={
            @Result(column = "id",property = "id" ),
            @Result(column = "building_id", property = "buildingId"),
            @Result(column = "cur_date", property = "curDate"),
            @Result(column = "env_min_temp", property = "envMinTemp"),
            @Result(column = "env_max_temp", property = "envMaxTemp"),
            @Result(column = "water_value", property = "waterValue"),
            @Result(column = "elect_value", property = "electValue"),
            @Result(column = "water_level", property = "waterLevel"),
            @Result(column = "water_value_pre", property = "waterValuePre"),
            @Result(column = "elect_value_pre", property = "electValuePre"),
            @Result(column = "water_level_pre", property = "waterLevelPre"),
            @Result(column = "remark", property = "remark")
    })
    @Select("select * from history_date_pre where building_id = #{buildingId} and cur_date = #{curDate} order by cur_date ")
    List<HistoryDataPre> getRecentData(@Param("buildingId") String buildingId,
                                       @Param("curDate") String curDate);

    @Update("update history_data_pre set water_value_pre = #{waterValuePre},elect_value_pre = #{electValuePre},water_level_pre = #{waterLevelPre}," +
            " water_value = #{waterValue},elect_value = #{electValue},water_level = #{waterLevel} " +
            " where id = #{id} and building_id = #{buildingId}")
    void updateById(HistoryDataPre preHistoryData);

    @Select("select count(*) from history_data_pre where building_id = #{buildingId} and cur_date = #{curDate} and env_min_temp is not null ")
    int selectIsPre(@Param("buildingId") String buildingId,
                    @Param("curDate") String curDate);

    @Results(id ="rs_cur_data",value ={
            @Result(column = "id",property = "id" ),
            @Result(column = "building_id", property = "buildingId"),
            @Result(column = "cur_date", property = "curDate"),
            @Result(column = "env_min_temp", property = "envMinTemp"),
            @Result(column = "env_max_temp", property = "envMaxTemp"),
            @Result(column = "water_value", property = "waterValue"),
            @Result(column = "elect_value", property = "electValue"),
            @Result(column = "water_level", property = "waterLevel"),
            @Result(column = "check_in_count", property = "peopleNum")
    })
    @Select("select top 1 " +
            "  convert(date,eds.cur_date) as cur_date, " +
            "  eds.building_id, " +
            "  isnull(eds.water_value, " +
            "  0) as water_value, " +
            "  isnull(eds.elect_value, " +
            "  0) as elect_value, " +
            "  isnull(convert(numeric(24, " +
            " 2), " +
            " t1.water_level), " +
            "  0) as water_level," +
            " b.check_in_count  " +
            "from " +
            "  energy_day_sum eds " +
            "left join ( " +
            " select " +
            "  convert(date, " +
            "  cur_date) as cur_date, " +
            "  building_id, " +
            "  avg(isnull(convert(numeric(24, 2), water_level), 0)) as water_level " +
            " from " +
            "  history_data " +
            " where " +
            "  building_id = #{buildingId} " +
            "  and cur_date = #{curDate} " +
            " group by " +
            "  convert(date, " +
            "  cur_date), " +
            "  building_id   " +
            " ) t1 on " +
            "  eds.cur_date = t1.cur_date " +
            " and eds.building_id = t1.building_id " +
            " left join building b " +
            " on b.id = eds.building_id " +
            "where " +
            " eds.building_id != '所有' " +
            " and eds.building_id = #{buildingId} " +
            " and eds.cur_date = #{curDate} " +
            "order by " +
            "  eds.building_id, " +
            "  eds.cur_date ")
    HistoryDataPre selectCurData(@Param("buildingId") String buildingId,
                                 @Param("curDate") String curDate);

    @Insert("insert into history_data_pre(cur_date, building_id, env_min_temp, env_max_temp, water_value, elect_value, water_level, people_num) values(" +
            "convert(date,#{curDate}), #{buildingId}, #{envMinTemp}, #{envMaxTemp}, #{waterValue}, #{electValue}, #{waterLevel}, #{peopleNum}" +
            ")")
    void insertData(HistoryDataPre curHistoryData);

    @Results(id ="rs_one_data",value ={
            @Result(column = "id",property = "id" ),
            @Result(column = "building_id", property = "buildingId"),
            @Result(column = "cur_date", property = "curDate"),
            @Result(column = "env_min_temp", property = "envMinTemp"),
            @Result(column = "env_max_temp", property = "envMaxTemp"),
            @Result(column = "people_num", property = "peopleNum")
    })
    @Select("select top 1 id, building_id, cur_date, env_min_temp, env_max_temp, people_num from history_data_pre where building_id = #{buildingId} and cur_date = #{curDate} ")
    HistoryDataPre selectOneData(@Param("buildingId") String buildingId,
                                 @Param("curDate") String curDate);

    @Results({
            @Result(column = "cur_yes_data",property = "curYesData" ),
            @Result(column = "yes_data", property = "yesData"),
            @Result(column = "pre_yes_data", property = "preYesData"),
            @Result(column = "error_data", property = "errorData")
    })
    @Select("<script>" +
            "SELECT " +
            "  <choose>" +
            "    <when test='type == \"2\"'>" +
            "      isnull(t.water_value_pre, 0) as cur_yes_data, " +
            "      isnull(t1.water_value, 0) as yes_data, " +
            "      isnull(t1.water_value_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.water_value_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(t1.water_value_pre, 0) - isnull(t1.water_value, 0)) / t1.water_value_pre * 100) ELSE '0' END as error_data" +
            "    </when>" +
            "    <when test='type == \"1\"'>" +
            "      isnull(t.elect_value_pre, 0) as cur_yes_data, " +
            "      isnull(t1.elect_value, 0) as yes_data, " +
            "      isnull(t1.elect_value_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.elect_value_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(t1.elect_value_pre, 0) - isnull(t1.elect_value, 0)) / t1.elect_value_pre * 100) ELSE '0' END as error_data" +
            "    </when>" +
            "    <when test='type == \"3\"'>" +
            "      isnull(t.water_level_pre, 0) as cur_yes_data, " +
            "      isnull(t1.water_level, 0) as yes_data, " +
            "      isnull(t1.water_level_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.water_level_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(t1.water_level_pre, 0) - isnull(t1.water_level, 0)) / t1.water_level_pre * 100) ELSE '0' END as error_data" +
            "    </when>" +
            "    <otherwise>null</otherwise>" +
            "  </choose>" +
            "FROM (" +
            "  SELECT " +
            "    building_id, " +
            "    water_value, " +
            "    elect_value, " +
            "    water_level, " +
            "    water_value_pre, " +
            "    elect_value_pre, " +
            "    water_level_pre " +
            "  FROM history_data_pre " +
            "  WHERE cur_date = CONVERT(varchar(10), GETDATE(), 120) AND building_id = #{buildingId}" +
            ") t " +
            "JOIN (" +
            "  SELECT " +
            "    building_id, " +
            "    water_value, " +
            "    elect_value, " +
            "    water_level, " +
            "    water_value_pre, " +
            "    elect_value_pre, " +
            "    water_level_pre " +
            "  FROM history_data_pre " +
            "  WHERE cur_date = CONVERT(varchar(10), DATEADD(day, -1, GETDATE()), 120) AND building_id = #{buildingId}" +
            ") t1 ON t.building_id = t1.building_id " +
            "</script>")
    List<EnergyPreTopDataDTO> getTopData(@Param("buildingId") String buildingId, @Param("type") String type);



    @Results({
            @Result(column = "cur_date",property = "curDate" ),
            @Result(column = "cur_data", property = "curData"),
            @Result(column = "pre_data", property = "preData"),
            @Result(column = "error_data", property = "errorData")
    })
    @Select("<script>" +
            "SELECT " +
            "  hdp.cur_date, " +
            "  <choose>" +
            "    <when test='type == \"2\"'>isnull(hdp.water_value, 0)</when>" +
            "    <when test='type == \"1\"'>isnull(hdp.elect_value, 0)</when>" +
            "    <when test='type == \"3\"'>isnull(hdp.water_level, 0)</when>" +
            "    <otherwise>0</otherwise>" +
            "  </choose> as cur_data, " +
            "  <choose>" +
            "    <when test='type == \"2\"'>isnull(hdp.water_value_pre, 0)</when>" +
            "    <when test='type == \"1\"'>isnull(hdp.elect_value_pre, 0)</when>" +
            "    <when test='type == \"3\"'>isnull(hdp.water_level_pre, 0)</when>" +
            "    <otherwise>0</otherwise>" +
            "  </choose> as pre_data, " +
            "  CASE " +
            "    <when test='type == \"2\"'>" +
            "      WHEN hdp.water_value_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(hdp.water_value_pre, 0) - isnull(hdp.water_value, 0)) / hdp.water_value_pre * 100) else 0 " +
            "    </when>" +
            "    <when test='type == \"1\"'>" +
            "      WHEN hdp.elect_value_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(hdp.elect_value_pre, 0) - isnull(hdp.elect_value, 0)) / hdp.elect_value_pre * 100) else 0 " +
            "    </when>" +
            "    <when test='type == \"3\"'>" +
            "      WHEN hdp.water_level_pre > 0 THEN CONVERT(decimal(18, 2), ABS(isnull(hdp.water_level_pre, 0) - isnull(hdp.water_level, 0)) / hdp.water_level_pre * 100) else 0 " +
            "    </when>" +
            "  END as error_data " +
            "FROM history_data_pre hdp " +
            "WHERE hdp.building_id = #{buildingId} " +
            "  AND cur_date BETWEEN #{beginDate} AND #{endDate} " +
            "ORDER BY cur_date" +
            "</script>")
    List<EnergyPreEchartDataDTO> getEnergyPre(@Param("buildingId") String buildingId,
                                              @Param("beginDate") String beginDate,
                                              @Param("endDate") String endDate,
                                              @Param("type") String type);
}
