package com.mh.user.serialport;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.factory.Device;
import com.mh.user.factory.DeviceFactory;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.NowDataService;
import com.mh.user.service.SysParamService;
import com.mh.user.strategy.DeviceStrategy;
import com.mh.user.strategy.DeviceStrategyFactory;
import com.mh.user.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import purejavacomm.SerialPort;

import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 通过串口发送和接收数据
 * @date 2024-03-18 14:56:32
 */
@Slf4j
public class SendAndReceiveByCom {

    List<DeviceCodeParamEntity> deviceManageEntityList;

    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    DeviceInstallService deviceInstallService = context.getBean(DeviceInstallService.class);
    NowDataService nowDataService = context.getBean(NowDataService.class);
    BuildingService buildingService = context.getBean(BuildingService.class);

    public void sendAndReceive(String sort, String thread) {
        SerialPort serialPort = null;
        CacheUtil cacheUtil = CacheUtil.getInstance();
        try {
            //生成对应的采集指令
            List<DeviceCodeParamEntity> deviceParamsByType = cacheUtil.getDeviceParamsByType(sort);
            deviceManageEntityList = deviceParamsByType
                    .parallelStream()
                    .filter(value -> value.getThread().equals(thread))
                    .sorted(Comparator.comparing(DeviceCodeParamEntity::getDataCom))
                    .collect(Collectors.toList());
            int size = deviceManageEntityList.size();
            for (int i = 0; i < size; i++) {
                //判断网页端是否有操作设备的
                if (Constant.WEB_FLAG) {
                    if (serialPort != null) {
                        SerialTool.closePort(serialPort);
                    }
                    log.info("有指令下发退出定时采集");
                    break;
                }
                String comName = deviceManageEntityList.get(i).getDataCom().toUpperCase();
                if (StringUtils.isBlank(comName)) {
                    log.info("-------------串口：" + comName + "不存在!-------------");
                    continue;
                }

                //获取设备实际波特率
                int baudrate = deviceManageEntityList.get(i).getBaudrate();
                //获取设备实际校验位
                String parity = deviceManageEntityList.get(i).getParity();
                String deviceAddr = deviceManageEntityList.get(i).getDeviceAddr();
                String deviceType = deviceManageEntityList.get(i).getDeviceType();
                String registerAddr = deviceManageEntityList.get(i).getRegisterAddr();
                String brand = deviceManageEntityList.get(i).getBrand();//品牌
                String buildingId = deviceManageEntityList.get(i).getBuildingId();
                String buildingName = buildingService.queryBuildingName(buildingId); //查询楼栋名称
                // 创建设备报文
                Device device = DeviceFactory.createDevice(deviceType);
                DeviceStrategy strategy = DeviceStrategyFactory.createStrategy(deviceType);
                if (null == strategy) {
                    continue;
                }
                device.setStrategy(strategy);
                String sendStr = device.createOrders(deviceManageEntityList.get(i));

                try {
                    //传入对应的串口参数并打开串口
                    if (StringUtils.isBlank(parity) || parity.equalsIgnoreCase("none")) {
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
                    } else {
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
                    }
                    if (null == serialPort) {
                        continue;
                    }
                    //向串口发送指令
                    if (StringUtils.isBlank(sendStr)) {
                        continue;
                    }
                    SerialTool.sendToPort(SerialTool.HexString2Bytes(sendStr), serialPort, sendStr, deviceType);
                    if (deviceType.equals("热泵")) {
                        for (int j = 0; j < 4; j++) {
                            Thread.sleep(1000);
                            // 判断网页端是否有操作设备的
                            if (Constant.WEB_FLAG) {
                                SerialTool.closePort(serialPort);
                                log.info("有指令下发退出定时采集");
                                break;
                            }
                        }
                    } else {
                        Thread.sleep(2000);
                    }

                    //从串口读取数据
                    byte[] bytes = SerialTool.readFromPort(serialPort);
                    Date date1 = new Date();
                    String dateStr = DateUtil.dateToString(date1, "yyyy-MM-dd HH:mm:ss");
                    if (bytes == null) {
                        SerialTool.closePort(serialPort);
                        log.info("串口" + serialPort.getName() + "没有数据返回！" + i);
                        log.info("----------------" + deviceType + "离线,设备号：" + deviceAddr + ",所属楼栋：" + buildingName + "----------------");
                        String time1 = deviceInstallService.selectLastDate(deviceType, deviceAddr, buildingId);
                        if (time1 == null) {
                            time1 = dateStr;
                        }
                        int d = ExchangeStringUtil.compareCopyTime(time1, dateStr);
                        if (d == 1) {
                            deviceInstallService.updateNotOnline(deviceAddr, deviceType, buildingId, "离线"); //所有设备离线
                            if (deviceType.equals("热泵")) {
                                nowDataService.updateRunState(buildingId, deviceAddr, "离线", buildingName); //监控界面状态表热泵在线状态
                            }
                        }
                        continue;
                    }
                    // 处理返回来的数据报文
                    dealReceiveData(dateStr, serialPort, i, deviceAddr, deviceType, registerAddr, brand, buildingId, buildingName, bytes, device);

                } catch (Exception e) {
                    if (null != serialPort) {
                        SerialTool.closePort(serialPort);
                        log.error("发送窗口数据异常==>", e);
                    }
                } finally {
                    if (null != serialPort) {
                        SerialTool.closePort(serialPort);
                        log.info("关闭串口==" + serialPort.getName());
                    }
                }
            }
        } catch (Exception e) {
            log.error("-------------串口采集异常！----------->>", e);
        }
    }

    /**
     * 处理返回来的数据
     *
     * @param dateStr
     * @param serialPort
     * @param i
     * @param deviceAddr
     * @param deviceType
     * @param registerAddr
     * @param brand
     * @param buildingId
     * @param buildingName
     * @param bytes
     * @throws InterruptedException
     */
    private void dealReceiveData(String dateStr,
                                 SerialPort serialPort,
                                 int i,
                                 String deviceAddr,
                                 String deviceType,
                                 String registerAddr,
                                 String brand,
                                 String buildingId,
                                 String buildingName, byte[] bytes, Device device) {
        try {
            String receiveStr = "";
            receiveStr = ExchangeStringUtil.parseByte2HexStr(bytes);
            //去掉空格和null
            receiveStr = receiveStr.replace("null", "");
            receiveStr = receiveStr.replace(" ", "");
            log.info("串口：" + serialPort.getName() + "，接受第" + i + "数据：" + receiveStr + "，大小： " + receiveStr.length());
            //返回值全部变成大写
            String receiveData = receiveStr.toUpperCase();
            //截取去掉FE
            String dataStr;
            if (receiveData.length() > 8) {
                String str1 = receiveData.substring(0, 8);
                String str2 = receiveData.substring(8);
                dataStr = str1.replace("FE", "") + str2;
            } else {
                dataStr = receiveData.replace("FE", "");
            }
            deviceInstallService.updateOnline(deviceAddr, deviceType, buildingId, "在线"); //设备在线
            log.info("----------------" + deviceType + "在线,设备号：" + deviceAddr + ",所属楼栋：" + buildingName + "----------------");
            if (deviceType.equals("热泵")) {
                String strState = nowDataService.selectState(buildingId, deviceAddr);
                if (strState != null && strState.equals("离线")) { //采集到数据
                    nowDataService.updateRunState(buildingId, deviceAddr, "不运行", buildingName); //监控界面状态表热泵在线状态
                }
            }
            // 解析返回来的数据
            device.analysisReceiveData(dateStr, deviceType, registerAddr, brand, buildingId, buildingName, dataStr);

        } catch (Exception e) {
            log.error("楼栋：" + buildingName + "设备类型：" + deviceType + "保存数据库失败！" + i, e);
        }
    }

}
