package com.mh.user.strategy;

import com.mh.common.utils.StringUtils;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.DataResultService;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.util.Date;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 电表运行策略
 * @date 2024-03-18 09:51:17
 */
@Slf4j
public class EleMeterStrategy implements DeviceStrategy {


    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    DataResultService dataResultService = context.getBean(DataResultService.class);

    private static class SingletonHolder {
        private static final EleMeterStrategy INSTANCE = new EleMeterStrategy();
    }

    private EleMeterStrategy() {
        // 防止外部直接实例化
    }

    public static EleMeterStrategy getInstance() {
        return EleMeterStrategy.SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrders(DeviceCodeParamEntity deviceCodeParamEntity) {
        String str = "";
        String deviceAddr = deviceCodeParamEntity.getDeviceAddr();
        if (deviceAddr != null && deviceAddr.length() > 0) {
            try {
                //0代表前面补充0,12代表长度为12,d代表参数为正数型,基表通讯号
                str = String.format("%012d", Long.parseLong(deviceAddr));
                //转换位置
                str = ExchangeStringUtil.changePosition(str);
                //拼接功能码
                str = "68" + str + "680102";
                String strData = "1090";
                //加33操作
                str = str + ExchangeStringUtil.addThree(strData);
                //检验和
                String checkSum = ExchangeStringUtil.makeChecksum(str);
                str = "FEFEFE" + str + checkSum + "16";
            } catch (Exception e) {
                log.error("生成采集电表指令错误==>", e);
            }
        }
        log.info("生成采集电表指令==>" + str);
        return str.toUpperCase();
    }

    @Override
    public String analysisReceiveData(String dateStr, String deviceType, String registerAddr, String brand, String buildingId, String buildingName, String dataStr) {
        String data = "";
        if (dataStr.length() == 36 || dataStr.length() == 40 || dataStr.length() == 44 || dataStr.length() == 50) {
            String checkStr = dataStr.substring(0, dataStr.length() - 4); //减去校验码
            String checkNum = ExchangeStringUtil.makeChecksum(checkStr);  //生成校验码
            //返回的校验码与重新生成的校验码进行校验
            if (checkNum.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4, dataStr.length() - 2))) {
                //表号，12位
                String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
                        + checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
                meterId = String.format("%012d", Long.parseLong(meterId));
                StringBuilder stringBuilder = new StringBuilder();
                if (dataStr.length() == 36) {
                    for (int i = 0; i < 4; i++) {
                        String data1 = checkStr.substring(32 - 2 * (i + 1), 32 - 2 * i);
                        stringBuilder.append(data1);
                    }
                } else {
                    for (int i = 0; i < 4; i++) {
                        String data1 = checkStr.substring(36 - 2 * (i + 1), 36 - 2 * i);
                        stringBuilder.append(data1);
                    }
                }
                data = stringBuilder.toString();
                data = ExchangeStringUtil.cutThree(data);
                // 0 代表前面补充0,4 代表长度为4,d 代表参数为正数型
                data = String.format("%08d", Long.parseLong(data));
                data = data.substring(0, 6) + "." + data.substring(6, 8);
                log.info("电表表号：" + meterId + ",电表读数：" + data);
                try {
                    DataResultEntity dataResultEntity = new DataResultEntity();
                    dataResultEntity.setDeviceAddr(meterId);//通讯编号
                    dataResultEntity.setDeviceType("电表");
                    dataResultEntity.setBuildingId(buildingId);
                    dataResultEntity.setCurValue(Double.parseDouble(data));     //当前读数
                    Date date = new Date();
                    dataResultEntity.setCurDate(date);           //当前日期
                    dataResultService.saveDataResult(dataResultEntity);
                    log.info("电表数据保存数据库成功！ 楼栋名称：" + buildingName);
                } catch (Exception e) {
                    log.error("电表数据保存数据库失败！楼栋名称：" + buildingName, e);
                }
            } else {
                log.info("电表报文检验失败: " + dataStr);
            }
        }
        if (!StringUtils.isBlank(data)) {
            data = String.valueOf(Double.valueOf(data)); //00010.76,去除读数前面带0的情况
        }
        return data;
    }
}
