package com.mh.user.strategy;

import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.NowPublicDataEntity;
import com.mh.user.service.*;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 电表运行策略
 * @date 2024-03-18 09:51:17
 */
@Slf4j
public class StatusCheckStrategy implements DeviceStrategy {

    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    NowDataService nowDataService = context.getBean(NowDataService.class);
    NowPublicDataService nowPublicDataService = context.getBean(NowPublicDataService.class);

    private static class SingletonHolder {
        private static final StatusCheckStrategy INSTANCE = new StatusCheckStrategy();
    }

    private StatusCheckStrategy() {
        // 防止外部直接实例化
    }

    public static StatusCheckStrategy getInstance() {
        return StatusCheckStrategy.SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrders(DeviceCodeParamEntity deviceCodeParamEntity) {
        String deviceAddr = deviceCodeParamEntity.getDeviceAddr();
        String brand = deviceCodeParamEntity.getBrand();
        String str = "";
        if (deviceAddr != null && deviceAddr.length() > 0) {
            try {
                // 0代表前面补充0,12代表长度为12,d代表参数为正数型.12位通讯地址,以0补位
                str = String.format("%012d", Long.parseLong(deviceAddr));
                // 转换位置
                str = ExchangeStringUtil.changePosition(str);
                // 拼接功能码
                str = "68" + str + "680102";
                String strData = "25C0";
                // 加33操作
                str = str + ExchangeStringUtil.addThree(strData);
                // 检验和
                String checkSum = ExchangeStringUtil.makeChecksum(str);
                str = "FEFEFE" + str + checkSum + "16";

            } catch (Exception e) {
                log.error("生成状态检测指令出错！" + str, e);
            }
        }
        return str.toUpperCase();
    }

    @Override
    public String analysisReceiveData(String dateStr, String deviceType, String registerAddr, String brand, String buildingId, String buildingName, String dataStr) {
        // 68 12 00 00 00 00 00 68 81 03 58 F3 35 E6 16
        if (dataStr.length() == 30) {
            //检验报文
            String checkStr = dataStr.substring(0, dataStr.length() - 4); //减去校验码
            String checkNum = ExchangeStringUtil.makeChecksum(checkStr);  //生成校验码
            //返回的校验码与重新生成的校验码进行校验
            if (!checkNum.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4, dataStr.length() - 2))) {
                return Constant.FAIL;
            }
            //表号，12位
            String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
                    + checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
            meterId = meterId.replaceAll("^0*", "");
            //数值
            String data = checkStr.substring(24, 26);
            data = ExchangeStringUtil.cutThree(data);
            String strStatus = ExchangeStringUtil.hexString2binaryString(data);
            strStatus = String.format("%08d", Long.parseLong(strStatus));
            String backWater;
            if (strStatus.charAt(7) == '1') {//回水
                backWater = "运行";
            } else {
                backWater = "不运行";
            }
            String hotPump;
            if (strStatus.charAt(6) == '1') {//热泵
                hotPump = "运行";
            } else {
                hotPump = "不运行";
            }
            String upWater;
            if (strStatus.charAt(5) == '1') {//上水或者补水
                upWater = "运行";
            } else {
                upWater = "不运行";
            }
            String useWater;
            if (strStatus.charAt(4) == '1') {//用水或者供水
                useWater = "运行";
            } else {
                useWater = "不运行";
            }
            NowPublicDataEntity publicData = new NowPublicDataEntity();
            publicData.setUseWater(useWater);
            publicData.setUpWater(upWater);
            publicData.setBackWater(backWater);
            publicData.setBuildingId(buildingId);
            try {
                //通过状态检测保存热泵状态
                nowDataService.saveNowHistoryData2(meterId, "状态检测", hotPump, "runState", buildingId);
                nowPublicDataService.saveNowHistoryPublicData(publicData);
                nowDataService.proPumpMinutes(buildingId, "1", hotPump); //如果只有一台热泵,pumpId为1
                log.info("状态ID：" + meterId + ",回水：" + backWater + ",补水：" + upWater + ",供水：" + useWater + ",热泵：" + hotPump + ",保存数据成功！楼栋：" + buildingName);
            } catch (Exception e) {
                log.error("解析状态检测异常==>", e);
            }
            return Constant.SUCCESS;
        }
        return Constant.FAIL;
    }
}
