package com.mh.user.strategy;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.NowDataService;
import com.mh.user.service.NowPublicDataService;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 时间控制器策略
 * @date 2024-03-18 09:51:17
 */
@Slf4j
public class TimeControlStrategy implements DeviceStrategy {


    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    NowDataService nowDataService = context.getBean(NowDataService.class);

    NowPublicDataService nowPublicDataService = context.getBean(NowPublicDataService.class);

    private static class SingletonHolder {
        private static final TimeControlStrategy INSTANCE = new TimeControlStrategy();
    }

    private TimeControlStrategy() {
        // 防止外部直接实例化
    }

    public static TimeControlStrategy getInstance() {
        return TimeControlStrategy.SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrders(DeviceCodeParamEntity deviceCodeParamEntity) {
        String deviceAddr = deviceCodeParamEntity.getDeviceAddr();
        String brand = deviceCodeParamEntity.getBrand();
        String funCode = deviceCodeParamEntity.getFunCode();
        String registerAddr = deviceCodeParamEntity.getRegisterAddr();
        String dataValue = deviceCodeParamEntity.getDataValue();
        String param = deviceCodeParamEntity.getParam();
        String str = "";
        if (deviceAddr != null && deviceAddr.length() > 0) {
            str = ExchangeStringUtil.decToHex(deviceAddr);
            str = ExchangeStringUtil.addZeroForNum(str, 2);  //以0补位
            try {
                if (!StringUtils.isBlank(funCode)) {
                    registerAddr = ExchangeStringUtil.addZeroForNum(registerAddr, 4); //寄存器地址
                    funCode = ExchangeStringUtil.addZeroForNum(funCode, 2);  // 功能码

                    if (funCode.equals("03")) { //功能码,读
                        if (param.equals("checkTime")) {
                            // 读取时间：0A 03 00 00 00 03 04 B0
                            str = str + "03" + registerAddr + "0003";
                        } else {
                            str = str + "03" + registerAddr + "0004";
                        }
                    } else if (funCode.equals("10")) {//写
                        if (!dataValue.equals("") && !param.equals("checkTime")) {
                            str = str + "10" + registerAddr + "000408" + dataValue;
                        } else {
                            // 校验时间：0A 10 00 00 00 03 06 00 24 01 10 08 59 48 CF
                            //         0A 10 00 00 00 03 06 00 24 01 10 09 40 8B 95
                            str = str + "10" + registerAddr + "00030600" + dataValue.substring(2);
                        }
                    }
                }
                String checkWord = ExchangeStringUtil.getStrCRC16(str); //CRC16校验
                str = str + checkWord;
            } catch (Exception e) {
                log.error("生成时控指令出错！" + str, e);
            }
        }
        return str.toUpperCase();
    }

    @Override
    public String analysisReceiveData(String dateStr, String deviceType, String registerAddr, String brand, String buildingId, String buildingName, String dataStr) {
        String result = Constant.FAIL;
        String checkStr = dataStr.substring(0, dataStr.length() - 4);
        String checkWord = ExchangeStringUtil.getStrCRC16(checkStr);//生成校验码
        if (!checkWord.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4))) {
            return result;
        }
        log.info("时控报文检验成功: " + dataStr);
        //地址
        String addr = ExchangeStringUtil.hexToDec(checkStr.substring(0, 2));
        log.info("时控号： " + addr);
        String data = "";
        if (checkStr.substring(2, 4).equalsIgnoreCase("03")) {// 读
            int rec = dataStr.length();

            if (rec == 26) {      //读时间设置值
                data = checkStr.substring(6, rec - 4);
                String curTimeBegin1 = data.substring(0, 2) + ":" + data.substring(2, 4);
                String curTimeEnd1 = data.substring(4, 6) + ":" + data.substring(6, 8);
                String curTimeBegin2 = data.substring(8, 10) + ":" + data.substring(10, 12);
                String curTimeEnd2 = data.substring(12, 14) + ":" + data.substring(14, 16);

            } else if (rec == 22) { //读校对时间
                data = checkStr.substring(8, rec - 4);
                String strDate = "20" + data.substring(0, 2) + "-" + data.substring(2, 4) + "-" + data.substring(4, 6); //拼接日期
                String strTime = data.substring(6, 8) + ":" + data.substring(8, 10);  //拼接时间
            }
            result = data;
        } else if (checkStr.substring(2, 4).equalsIgnoreCase("05") || checkStr.substring(2, 4).equalsIgnoreCase("10")) {
            log.info("时间： " + dateStr + "，时控ID： " + addr + "，操作成功！ ");
            return Constant.SUCCESS;
        }
        return result;
    }
}
