package com.mh.user.strategy;

import com.alibaba.fastjson2.JSON;
import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.entity.NowPublicDataEntity;
import com.mh.user.entity.SysParamEntity;
import com.mh.user.service.*;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.util.HashMap;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project CHWS
 * @description 水位开关策略
 * @date 2024-03-18 09:51:17
 */
@Slf4j
public class WaterLevelSwitchStrategy implements DeviceStrategy {


    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    NowDataService nowDataService = context.getBean(NowDataService.class);

    NowPublicDataService nowPublicDataService = context.getBean(NowPublicDataService.class);

    SysParamService sysParamService = context.getBean(SysParamService.class);

    BuildingService buildingService = context.getBean(BuildingService.class);

    DeviceInstallService deviceInstallService = context.getBean(DeviceInstallService.class);

    private static class SingletonHolder {
        private static final WaterLevelSwitchStrategy INSTANCE = new WaterLevelSwitchStrategy();
    }

    private WaterLevelSwitchStrategy() {
        // 防止外部直接实例化
    }

    public static WaterLevelSwitchStrategy getInstance() {
        return WaterLevelSwitchStrategy.SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrders(DeviceCodeParamEntity deviceCodeParamEntity) {
        String deviceAddr = deviceCodeParamEntity.getDeviceAddr();
        String brand = deviceCodeParamEntity.getBrand();
        String funCode = deviceCodeParamEntity.getFunCode();
        String registerAddr = deviceCodeParamEntity.getRegisterAddr();
        String dataValue = deviceCodeParamEntity.getDataValue();
        String str = "";
        if (deviceAddr != null && deviceAddr.length() > 0) {
            try {
                if (StringUtils.isBlank(brand) || Constant.BRAND_ZHONG_KAI.equals(brand)) {
                    str = ExchangeStringUtil.decToHex(deviceAddr);
                    str = ExchangeStringUtil.addZeroForNum(str, 2);
                    if (StringUtils.isBlank(funCode) || funCode.equals("17")) { //读
                        str = "33" + str + "1700000000";  //旧版继电器
                    } else if (funCode.equals("12")) {//写
                        if (dataValue != null && !dataValue.equals("")) {
                            //0001(90%) 0002(80%) 0003(70%)...
                            if (dataValue.equals("100%")) {
                                registerAddr = "1300000000";  //全开
                            } else if (dataValue.equals("90%")) {
                                registerAddr = "1200000001";
                            } else if (dataValue.equals("80%")) {
                                registerAddr = "1200000002";
                            } else if (dataValue.equals("70%")) {
                                registerAddr = "1200000003";
                            } else if (dataValue.equals("60%")) {
                                registerAddr = "1200000004";
                            } else if (dataValue.equals("50%")) {
                                registerAddr = "1200000005";
                            } else if (dataValue.equals("40%")) {
                                registerAddr = "1200000006";
                            }
                            str = "33" + str + registerAddr;  //示例：33011200000001
                        }
                    } else if (funCode.equals("13")) {//写
                        str = "33" + str + "1300000000";  //全开
                    } else {
                        str = "33" + str + "1400000000";  //全合
                    }
                    // 检验和
                    String checkSum = ExchangeStringUtil.makeChecksum(str);
                    str = str + checkSum;

                } else if (Constant.BRAND_YUAN_XIANG.equals(brand)) {
                    str = ExchangeStringUtil.decToHex(deviceAddr);
                    str = ExchangeStringUtil.addZeroForNum(str, 2);
                    if (StringUtils.isBlank(funCode) || funCode.equals("03")) {//读
                        if (registerAddr.equalsIgnoreCase("0018")) {            //读全部位置
                            str = str + "0300180008";
                        } else if (registerAddr.equalsIgnoreCase("0010")) {     //读全部DI
                            str = str + "0300100008";
                        }
                    } else if (funCode.equals("06")) { // 写
                        if (!StringUtils.isBlank(dataValue)) {
                            if (dataValue.equals("100%")) {
                                registerAddr = "10001800081000000000000000000000000000000000";  //全开
                            } else if (dataValue.equals("90%")) {
                                registerAddr = "0600180001";
                            } else if (dataValue.equals("80%")) {
                                registerAddr = "0600190001";
                            } else if (dataValue.equals("70%")) {
                                registerAddr = "06001A0001";
                            } else if (dataValue.equals("60%")) {
                                registerAddr = "06001B0001";
                            } else if (dataValue.equals("50%")) {
                                registerAddr = "06001C0001";
                            } else if (dataValue.equals("40%")) {
                                registerAddr = "06001D0001";
                            }
                            //0018(90%) 0019(80%) 001A(70%)...
                            str = str + registerAddr;  //示例：010600180001
                        }
                    } else {//写
                        if (dataValue.equals("0000")) {
                            str = str + "10001800081000000000000000000000000000000000";  //全开
                        } else {
                            str = str + "10001800081000010001000100010001000100010001";  //全合
                        }
                    }
                    String checkWord = ExchangeStringUtil.getStrCRC16(str); //CRC16校验
                    str = str + checkWord;
                } else if (Constant.BRAND_DING_WEI.equals(brand)) {
                    str = String.format("%012d", Long.parseLong(deviceAddr));
                    //转换位置
                    str = ExchangeStringUtil.changePosition(str); //表号
                    String strData = "";
                    if (funCode.equals("0407")) {//写
                        str = "68" + str + "680407";
                        String mark = "25C0";          //C025交换位置
                        String strData1 = "00000000";  //命令带密码4字节,权限密码默认为00000000
                        String strData2 = "";
                        if (dataValue.equals("100%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00000000");
                        } else if (dataValue.equals("90%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00000001");
                        } else if (dataValue.equals("85%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00000011");
                        } else if (dataValue.equals("80%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00000111");
                        } else if (dataValue.equals("75%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00001111");
                        } else if (dataValue.equals("70%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00011111");
                        } else if (dataValue.equals("65%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00111111");
                        } else if (dataValue.equals("60%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("01111111");
                        } else if (dataValue.equals("50%")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("11111111");
                        } else if (dataValue.equals("自动")) {
                            strData2 = ExchangeStringUtil.parseByte2HexStr("00100110");
                        }
                        strData = mark + strData1 + strData2;
                    } else if (funCode.equals("0102")) { //读
                        str = "68" + str + "680102";
                        strData = "25C0"; //标志
                    }
                    strData = ExchangeStringUtil.addThree(strData); //加33
                    str = str + strData;
                    //检验和
                    String checkSum = ExchangeStringUtil.makeChecksum(str);//校验
                    str = "FEFEFE" + str + checkSum + "16";
                }
            } catch (Exception e) {
                if (registerAddr.equals("0010")) {
                    log.error("生成热泵状态指令出错！" + str, e);
                } else {
                    log.error("生成水位开关指令出错！" + str, e);
                }
            }
        }
        return str.toUpperCase();
    }

    @Override
    public String analysisReceiveData(String dateStr, String deviceType, String registerAddr, String brand, String buildingId, String buildingName, String dataStr) {
        SysParamEntity sysParamEntity = sysParamService.selectSysParam();
        String customName = sysParamEntity.getCustomName();
        if (StringUtils.isBlank(registerAddr) || Constant.REG_ADDR_0018.equals(registerAddr) || Constant.REG_ADDR_0017.equals(registerAddr)) {
            return analysisRelayOrder485(dateStr, dataStr, registerAddr, brand, buildingId, buildingName, customName);
        } else if (Constant.REG_ADDR_0010.equals(registerAddr) && !StringUtils.isBlank(registerAddr)){
            return handleWaterSwitch(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
        }
        return Constant.FAIL;
    }

    private String handleWaterSwitch(String dateStr, String dataStr, String registerAddr, String brand,
                                   String buildingId, String buildingName) {
        log.info("进入水位开关，地址==>{}", registerAddr);
        SysParamEntity sysParamEntity = sysParamService.selectSysParam();
        String customName = sysParamEntity.getCustomName();
        if (!StringUtils.isBlank(customName)) {
            if ((customName.contains(Constant.CUSTOM_NAME_HUAXIA)
                    || customName.contains(Constant.CUSTOM_NAME_GUANGSHANG)) && !Constant.WEB_FLAG ) {
                analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
                return analysisRelayOrder485(dateStr, dataStr,registerAddr,brand,buildingId, buildingName, customName);
            } else {
                return analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
            }
        } else {
            return analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
        }
    }

    // 热泵状态处理(水位开关),长度42,128
    public String analysisPumpStateOrder(final String dateStr, final String receiveData, final String registerAddr, final String brand, String buildingId, String buildingName) {
        String result = Constant.FAIL;
        String l1 = "";
        String l2 = "";
        String l3 = "";
        String l4 = "";
        String l5 = "";
        String l6 = "";
        String l7 = "";
        String l8 = "";
        String runState = "";
        String addr = "";
        Map<String, String> map = new HashMap<String, String>();
        if (StringUtils.isBlank(brand) || brand.equals("顶威")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4);  //检验报文减,去校验码
            String checkNum = ExchangeStringUtil.makeChecksum(checkStr);           //生成校验码
            if (checkNum.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4, receiveData.length() - 2))) {
                //表号，12位
                addr = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
                        + checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
                addr = addr.replaceAll("^0*", "");
                String data = checkStr.substring(24, 26); //截取数值
                data = ExchangeStringUtil.cutThree(data);
                String strStatus = ExchangeStringUtil.hexString2binaryString(data);
                strStatus = String.format("%08d", Long.parseLong(strStatus));
                if (strStatus.charAt(7) == '1') {
                    l1 = "1";
                } else {
                    l1 = "0";
                }
                if (strStatus.charAt(6) == '1') {
                    l2 = "1";
                } else {
                    l2 = "0";
                }
                if (strStatus.charAt(5) == '1') {
                    l3 = "1";
                } else {
                    l3 = "0";
                }
                if (strStatus.charAt(4) == '1') {
                    l4 = "1";
                } else {
                    l4 = "0";
                }
                if (strStatus.charAt(3) == '1') {
                    l5 = "1";
                } else {
                    l5 = "0";
                }
                if (strStatus.charAt(2) == '1') {
                    l6 = "1";
                } else {
                    l6 = "0";
                }
                if (strStatus.charAt(1) == '1') {
                    l7 = "1";
                } else {
                    l7 = "0";
                }
                if (strStatus.charAt(0) == '1') {
                    l8 = "1";
                } else {
                    l8 = "0";
                }
            }
        } else if (brand.equals("中凯")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4); //检验报文,减去校验码
            addr = ExchangeStringUtil.hexToDec(checkStr.substring(2, 4));
            l1 = checkStr.substring(4, 6);
            l2 = checkStr.substring(20, 22);
            l3 = checkStr.substring(36, 38);
            l4 = checkStr.substring(52, 54);
            l5 = checkStr.substring(68, 70);
            l6 = checkStr.substring(84, 86);
            l7 = checkStr.substring(100, 102);
            l8 = checkStr.substring(116, 118);
        } else if (brand.equals("远向")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4);// 检验报文
            String checkWord = ExchangeStringUtil.getStrCRC16(checkStr);
            if (checkWord.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4))) {
                addr = ExchangeStringUtil.hexToDec(checkStr.substring(0, 2));//地址
                l1 = checkStr.substring(6, 10);
                l2 = checkStr.substring(10, 14);
                l3 = checkStr.substring(14, 18);
                l4 = checkStr.substring(18, 22);
                l5 = checkStr.substring(22, 26);
                l6 = checkStr.substring(26, 30);
                l7 = checkStr.substring(30, 34);
                l8 = checkStr.substring(34, 38);
            } else {
                log.info("热泵状态报文检验失败: " + receiveData);
            }
        }
        if (l1.equals("12") || l1.equals("0001")) { //正常是0001
            map.put("1", "运行");
        } else {
            map.put("1", "不运行");
        }
        if (l2.equals("12") || l2.equals("0001")) {
            map.put("2", "运行");
        } else {
            map.put("2", "不运行");
        }
        if (l3.equals("12") || l3.equals("0001")) {
            map.put("3", "运行");
        } else {
            map.put("3", "不运行");
        }
        if (l4.equals("12") || l4.equals("0001")) {
            map.put("4", "运行");
        } else {
            map.put("4", "不运行");
        }
        if (l5.equals("12") || l5.equals("0001")) {
            map.put("5", "运行");
        } else {
            map.put("5", "不运行");
        }
        if (l6.equals("12") || l6.equals("0001")) {
            map.put("6", "运行");
        } else {
            map.put("6", "不运行");
        }
        if (l7.equals("12") || l7.equals("0001")) {
            map.put("7", "运行");
        } else {
            map.put("7", "不运行");
        }
        if (l8.equals("12") || l8.equals("0001")) {
            map.put("8", "运行");
        } else {
            map.put("8", "不运行");
        }
        int count = buildingService.selectPumpCount(buildingId); //楼栋热泵数
        if (count > 0) {
            for (Map.Entry<String, String> entry : map.entrySet()) {
                addr = entry.getKey(); //获取Key值作为热泵ID
                runState = entry.getValue();
                if (addr != null && Integer.parseInt(addr) <= count) {
                    nowDataService.saveNowHistoryData3(addr, "热泵状态", runState, "runState", buildingId);
                    nowDataService.proPumpMinutes(buildingId, addr, runState); //计算热泵运行时长
                    nowDataService.proWaterTemp(dateStr, buildingId, addr);    //保存时间点温度
                    log.info("计算热泵运行时长,楼栋名称:" + buildingName + ",热泵编号:" + addr + ",状态:" + runState);
                }
            }
            log.info("热泵状态号：" + addr + ",保存数据库成功！楼栋名称：" + buildingName);
            result = JSON.toJSONString(map); //map转json字符串
        }
        return result;
    }

    // 水位开关,长度42,128
    public String analysisRelayOrder485(final String dateStr,final String receiveData, final String registerAddr, final String brand, String buildingId, String buildingName, String customName) {
//		threadPoolService.execute(() -> {
        String l1 = "";
        String l2 = "";
        String l3 = "";
        String l4 = "";
        String l5 = "";
        String l6 = "";
        String l7 = "";
        String l8 = "";
        String waterLevelSet = "";
        String addr = "";
        if (brand == null || brand.equals("") || brand.equals("中凯")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4); //减去校验码
            addr = ExchangeStringUtil.hexToDec(checkStr.substring(2, 4));
            if (registerAddr != null && !registerAddr.equals("")) {
                if (registerAddr.equalsIgnoreCase("0017")) {      //中凯拿功能码作为寄存器地址用
                    l1 = checkStr.substring(4, 6);
                    l2 = checkStr.substring(20, 22);
                    l3 = checkStr.substring(36, 38);
                    l4 = checkStr.substring(52, 54);
                    l5 = checkStr.substring(68, 70);
                    l6 = checkStr.substring(84, 86);
                    l7 = checkStr.substring(100, 102);
                    l8 = checkStr.substring(116, 118);
                } else if (registerAddr.equals("0012")) {
                    log.info("时间：" + dateStr + "，水位开关ID：" + addr + "，操作成功！ ");
                }
            }
        } else if (brand.equals("远向")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4);// 检验报文
            String checkWord = ExchangeStringUtil.getStrCRC16(checkStr);
            if (checkWord.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4))) {
                addr = ExchangeStringUtil.hexToDec(checkStr.substring(0, 2));//地址
                if (checkStr.substring(2, 4).equals("03")) {
                    if (registerAddr != null && !registerAddr.equals("")) {
//                        if (registerAddr.equals("0018")){
                            l1 = checkStr.substring(6, 10);
                            l2 = checkStr.substring(10, 14);
                            l3 = checkStr.substring(14, 18);
                            l4 = checkStr.substring(18, 22);
                            l5 = checkStr.substring(22, 26);
                            l6 = checkStr.substring(26, 30);
                            l7 = checkStr.substring(30, 34);
                            l8 = checkStr.substring(34, 38);
//                        }
                    }
                } else if (checkStr.substring(2, 4).equals("06") || checkStr.substring(2, 4).equals("04") || checkStr.substring(2, 4).equals("10")) {
                    log.info("时间：" + dateStr + "，水位开关ID：" + addr + "，操作成功！ ");
                    return Constant.SUCCESS;
                }
            } else {
                log.info("水位开关报文检验失败: " + receiveData);
            }
        } else if (brand.equals("顶威")) {
            String checkStr = receiveData.substring(0, receiveData.length() - 4); //减去校验码
            String checkNum = ExchangeStringUtil.makeChecksum(checkStr);          //生成校验码
            if (checkNum.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4, receiveData.length() - 2))) {
                //表号，12位
                String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
                        + checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
                meterId = String.format("%012d", Long.parseLong(meterId));
                if (registerAddr != null && !registerAddr.equals("")) {
                    if (registerAddr.equals("0102")) {
                        String data = receiveData.substring(26, 28);
                        data = ExchangeStringUtil.cutThree(data);
                        data = ExchangeStringUtil.addZeroForNum(data, 2);
                        if (data.equalsIgnoreCase("00")) {//全闭
                            waterLevelSet = "100"; //100%
                        } else if (data.equalsIgnoreCase("01")) {
                            waterLevelSet = "90";  //90%
                        } else if (data.equalsIgnoreCase("03")) {
                            waterLevelSet = "85";
                        } else if (data.equalsIgnoreCase("07")) {
                            waterLevelSet = "80";
                        } else if (data.equalsIgnoreCase("0F")) {
                            waterLevelSet = "75";
                        } else if (data.equalsIgnoreCase("1F")) {
                            waterLevelSet = "70";
                        } else if (data.equalsIgnoreCase("3F")) {
                            waterLevelSet = "65";
                        } else if (data.equalsIgnoreCase("7F")) {
                            waterLevelSet = "60";
                        } else if (data.equalsIgnoreCase("FF")) { //全开
                            waterLevelSet = "50";
                        } else if (data.equalsIgnoreCase("24")) {
                            waterLevelSet = "自动";
                        }
                        nowDataService.saveNowHistoryData2(meterId, "水位开关", waterLevelSet, "levelSet", buildingId);
                        log.info("水位开关号：" + meterId + ",保存数据库成功！楼栋名称：" + buildingName);
                    } else if (registerAddr.equalsIgnoreCase("0407")) {
                        log.info("时间： " + dateStr + "，继电器ID： " + meterId + "，操作成功！ ");
                    }
                }
            } else {
                log.info("水位开关报文检验失败: " + receiveData);
            }
        }
        if (registerAddr != null && !registerAddr.equals("")) {
            if (registerAddr.equals("0017") || registerAddr.equals("0018")) {
                if (l1.equals("12") || l1.equals("0001")) {
                    waterLevelSet = "90";
                } else if (l2.equals("12") || l2.equals("0001")) {
                    waterLevelSet = "80";
                } else if (l3.equals("12") || l3.equals("0001")) {
                    waterLevelSet = "70";
                } else if (l4.equals("12") || l4.equals("0001")) {
                    waterLevelSet = "60";
                } else if (l5.equals("12") || l5.equals("0001")) {
                    waterLevelSet = "50";
                } else if (l6.equals("12") || l6.equals("0001")) {
                    waterLevelSet = "40";
                }
                //全开或者全关
                if (l1.equals("12") && l2.equals("12") && l3.equals("12") && l4.equals("12") && l5.equals("12") && l6.equals("12")) {
                    waterLevelSet = "40";
                } else if (l1.equals("11") && l2.equals("11") && l3.equals("11") && l4.equals("11") && l5.equals("11") && l6.equals("11")) {
                    waterLevelSet = "100";
                } else if (l1.equals("0001") && l2.equals("0001") && l3.equals("0001") && l4.equals("0001") && l5.equals("0001") && l6.equals("0001")) {
                    waterLevelSet = "40";
                } else if (l1.equals("0000") && l2.equals("0000") && l3.equals("0000") && l4.equals("0000") && l5.equals("0000") && l6.equals("0000")) {
                    waterLevelSet = "100";
                }
                nowDataService.saveNowHistoryData2(addr, "水位开关", waterLevelSet, "levelSet", buildingId);
                log.info("水位开关保存数据库成功！楼栋名称：" + buildingName);
                return waterLevelSet;
            } else if (registerAddr.equals("0010")) {
                NowPublicDataEntity publicData = new NowPublicDataEntity();
                publicData.setBuildingId(buildingId);
                if (customName.contains(Constant.CUSTOM_NAME_GUANGSHANG)) {
                    if (l8.equals("0001")) {  //l8路，0000表示有输入，0001表示无输入
                        publicData.setUseWater("运行"); //供水
                        publicData.setBackWater("运行");//回水
                    } else {
                        publicData.setUseWater("不运行");
                        publicData.setBackWater("不运行");
                    }
                    log.info("-----------------供水/回水状态：" + l8 + "-----------------");
                } else {
                    if (l2.equals("0001") || l3.equals("0001")) {  //2,3路，0000表示有输入，0001表示无输入
                        publicData.setUseWater("运行"); //供水
                        publicData.setBackWater("运行");//回水
                    } else {
                        publicData.setUseWater("不运行");
                        publicData.setBackWater("不运行");
                    }
                    log.info("-----------------供水/回水状态：" + l2 + "--" + l3 + "-----------------");
                    if (l1.equals("0001")) {  //1路  //补水
                        publicData.setUpWater("运行");
                    } else {
                        publicData.setUpWater("不运行");
                    }
                    log.info("-----------------补水状态：" + l1 + "-----------------");
                }
                nowPublicDataService.saveNowHistoryPublicData(publicData);
                log.info("地址：{}，供回补水状态保存数据库成功！楼栋ID：{}，名称：{}" ,addr, buildingId, buildingName);
                // 更新now_data表中对应的供水、回水、补水状态
                DeviceInstallEntity deviceInstallEntity = deviceInstallService.selectDevice(addr, "水位开关", buildingId);
                if (deviceInstallEntity != null
                        && !StringUtils.isBlank(deviceInstallEntity.getDeviceName())) {
                    if (deviceInstallEntity.getDeviceName().contains("低")) {
                        log.info("更新now_data表中低区对应的供水、回水、补水状态，楼栋ID：{}，供水状态：{}，回水状态：{}，补水状态：{}", buildingId, publicData.getUseWater(), publicData.getBackWater(), publicData.getUpWater());
                        nowDataService.updateNowDataByPumpName("低", buildingId, publicData.getUseWater(), publicData.getBackWater(), publicData.getUpWater());
                    } else if (deviceInstallEntity.getDeviceName().contains("高")) {
                        log.info("更新now_data表中高区对应的供水、回水、补水状态，楼栋ID：{}，供水状态：{}，回水状态：{}，补水状态：{}", buildingId, publicData.getUseWater(), publicData.getBackWater(), publicData.getUpWater());
                        nowDataService.updateNowDataByPumpName("高", buildingId, publicData.getUseWater(), publicData.getBackWater(), publicData.getUpWater());
                    } else {
                        nowDataService.updateNowDataByPumpName(null, buildingId, publicData.getUseWater(), publicData.getBackWater(), publicData.getUpWater());
                    }
                    // 判断今天的记录是否存在多个补水，供水，回水状态值
                    Map<String, Object> states = nowDataService.selectTopOneState(buildingId);
                    if (states != null && !states.isEmpty()) {
                        publicData.setUpWater(String.valueOf(states.get("up_water_state")));
                        publicData.setUseWater(String.valueOf(states.get("use_water_state")));
                        publicData.setBackWater(String.valueOf(states.get("back_water_state")));
                    }
                    nowDataService.updateNowPublicData(publicData);
                }
            }
        }
        return Constant.FAIL;
    }
}
